/*
 * Copyright 2020 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

#include <stdlib.h>
#include <string.h>

#include <jpidef.h>
#include <starlet.h>

typedef short int _int16;
typedef long int _int32;
typedef long long int _int64;

struct procinfo
{
    _int64 frep0va;
    _int64 frep1va;
    _int64 last_login_i;
    _int64 last_login_n;
    _int64 logintim;
    _int64 p0_first_free_va_64;
    _int64 p1_first_free_va_64;
    _int64 p2_first_free_va_64;
    _int64 virtpeak;
    _int32 astcnt;
    _int32 astlm;
    _int32 biocnt;
    _int32 biolm;
    _int32 bufio;
    _int32 bytcnt;
    _int32 bytlm;
    _int32 cpulim;
    _int32 cputim;
    _int32 diocnt;
    _int32 diolm;
    _int32 dirio;
    _int32 enqcnt;
    _int32 enqlm;
    _int32 filcnt;
    _int32 fillm;
    _int32 gpgcnt;
    _int32 grp;
    _int32 imagecount;
    _int32 mem;
    _int32 pageflts;
    _int32 pgflquota;
    _int32 ppgcnt;
    _int32 prccnt;
    _int32 prclm;
    _int32 shrfillm;
    _int32 tqcnt;
    _int32 tqlm;
    _int32 uic;
    _int32 wsauth;
    _int32 wsextent;
    _int32 wspeak;
    _int32 wsquota;
    _int32 wssize;
    char account[8];
    char prcnam[16];
    char terminal[16];
    char username[12];
    char imagname[256];
};

struct item
{
    _int16 buflen;
    _int16 code;
    void *bufadr;
    void *retlenadr;
};

#define ITEM(code,res) { sizeof(res), code, &res, NULL }

static struct procinfo pi;

static struct item itmlst[] =
{
    ITEM(JPI$_ACCOUNT, pi.account),
    ITEM(JPI$_ASTCNT, pi.astcnt),
    ITEM(JPI$_ASTLM, pi.astlm),
    ITEM(JPI$_BIOCNT, pi.biocnt),
    ITEM(JPI$_BIOLM, pi.biolm),
    ITEM(JPI$_BUFIO, pi.bufio),
    ITEM(JPI$_BYTCNT, pi.bytcnt),
    ITEM(JPI$_BYTLM, pi.bytlm),
    ITEM(JPI$_CPULIM, pi.cpulim),
    ITEM(JPI$_CPUTIM, pi.cputim),
    ITEM(JPI$_DIOCNT, pi.diocnt),
    ITEM(JPI$_DIOLM, pi.diolm),
    ITEM(JPI$_DIRIO, pi.dirio),
    ITEM(JPI$_ENQCNT, pi.enqcnt),
    ITEM(JPI$_ENQLM, pi.enqlm),
    ITEM(JPI$_FILCNT, pi.filcnt),
    ITEM(JPI$_FILLM, pi.fillm),
    ITEM(JPI$_FREP0VA, pi.frep0va),
    ITEM(JPI$_FREP1VA, pi.frep1va),
    ITEM(JPI$_GPGCNT, pi.gpgcnt),
    ITEM(JPI$_GRP, pi.grp),
    ITEM(JPI$_IMAGECOUNT, pi.imagecount),
    ITEM(JPI$_IMAGNAME, pi.imagname),
    ITEM(JPI$_LAST_LOGIN_I, pi.last_login_i),
    ITEM(JPI$_LAST_LOGIN_N, pi.last_login_n),
    ITEM(JPI$_LOGINTIM, pi.logintim),
    ITEM(JPI$_MEM, pi.mem),
    ITEM(JPI$_PAGEFLTS, pi.pageflts),
    ITEM(JPI$_PGFLQUOTA, pi.pgflquota),
    ITEM(JPI$_P0_FIRST_FREE_VA_64, pi.p0_first_free_va_64),
    ITEM(JPI$_P1_FIRST_FREE_VA_64, pi.p1_first_free_va_64),
    ITEM(JPI$_P2_FIRST_FREE_VA_64, pi.p2_first_free_va_64),
    ITEM(JPI$_PPGCNT, pi.ppgcnt),
    ITEM(JPI$_PRCCNT, pi.prccnt),
    ITEM(JPI$_PRCLM, pi.prclm),
    ITEM(JPI$_PRCNAM, pi.prcnam),
    ITEM(JPI$_SHRFILLM, pi.shrfillm),
    ITEM(JPI$_TERMINAL, pi.terminal),
    ITEM(JPI$_TQCNT, pi.tqcnt),
    ITEM(JPI$_TQLM, pi.tqlm),
    ITEM(JPI$_UIC, pi.uic),
    ITEM(JPI$_USERNAME, pi.username),
    ITEM(JPI$_VIRTPEAK, pi.virtpeak),
    ITEM(JPI$_WSAUTH, pi.wsauth),
    ITEM(JPI$_WSEXTENT, pi.wsextent),
    ITEM(JPI$_WSPEAK, pi.wspeak),
    ITEM(JPI$_WSQUOTA, pi.wsquota),
    ITEM(JPI$_WSSIZE, pi.wssize),
};

#include "jpi.h"

int get_jpi_data(long int pid, char *buf, int buflen)
{
    memset(&pi, 0, sizeof(pi));
    sys$getjpiw(0, &pid, 0, itmlst, 0, 0, 0);
    int len = (buflen >= sizeof(pi)) ? sizeof(pi) : buflen;
    memcpy(buf, &pi, len);
    return len;
}

