/*
 * Copyright 2020 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

#include <stdlib.h>
#include <string.h>

#include <syidef.h>
#include <starlet.h>

typedef short int _int16;
typedef long int _int32;
typedef long long int _int64;

struct sysinfo
{
    _int64 boottime;
    _int64 pt_base;
    _int32 arch_type;
    _int32 contig_gblpages;
    _int32 cputype;
    _int32 free_gblpages;
    _int32 free_gblsects;
    _int32 memsize;
    _int32 node_number;
    _int32 pagefile_free;
    _int32 pagefile_page;
    _int32 serial_number;
    _int32 swapfile_free;
    _int32 swapfile_page;
    _int16 cluster_votes;
    _int16 hw_model;
    char boot_device[16];
    char nodename[16];
    char version[8];
};

struct item
{
    _int16 buflen;
    _int16 code;
    void *bufadr;
    void *retlenadr;
};

#define ITEM(code,res) { sizeof(res), code, &res, NULL }

static struct sysinfo si;

static struct item itmlst[] =
{
    ITEM(SYI$_ARCH_TYPE, si.arch_type),
    ITEM(SYI$_BOOT_DEVICE, si.boot_device),
    ITEM(SYI$_BOOTTIME, si.boottime),
    ITEM(SYI$_CLUSTER_VOTES, si.cluster_votes),
    ITEM(SYI$_CONTIG_GBLPAGES, si.contig_gblpages),
    ITEM(SYI$_CPUTYPE, si.cputype),
    ITEM(SYI$_FREE_GBLPAGES, si.free_gblpages),
    ITEM(SYI$_FREE_GBLSECTS, si.free_gblsects),
    ITEM(SYI$_HW_MODEL, si.hw_model),
    ITEM(SYI$_MEMSIZE, si.memsize),
    ITEM(SYI$_NODE_NUMBER, si.node_number),
    ITEM(SYI$_NODENAME, si.nodename),
    ITEM(SYI$_PAGEFILE_FREE, si.pagefile_free),
    ITEM(SYI$_PAGEFILE_PAGE, si.pagefile_page),
    ITEM(SYI$_PT_BASE, si.pt_base),
    ITEM(SYI$_SERIAL_NUMBER, si.serial_number),
    ITEM(SYI$_SWAPFILE_FREE, si.swapfile_free),
    ITEM(SYI$_SWAPFILE_PAGE, si.swapfile_page),
    ITEM(SYI$_VERSION, si.version)
};

#include "syi.h"

int get_syi_data(char *buf, int buflen)
{
    memset(&si, 0, sizeof(si));
    sys$getsyiw(0, 0, 0, itmlst, 0, 0, 0);
    int len = (buflen >= sizeof(si)) ? sizeof(si) : buflen;
    memcpy(buf, &si, len);
    return len;
}

